require 'mechanize'
require 'json'
require 'hashie'
require 'bigdecimal'

module Mintpal
  
  VERSION = '0.0.10'
  
  API_URL = 'https://api.mintpal.com/market'
  VALID_EXCHANGES = [
                      :BTC, :LTC
                    ]
  VALID_COINS     = [
                      :AUR, :BC, :BTCS, :CAGE, :CTM, :DGB, :DOGE, :DOPE, :DRK, :ECC, :EMO,
                      :FLT, :HVC, :KARM, :KDC, :LTC, :MINT, :MRC, :MRS, :MZC, :OLY, :PANDA, 
                      :PENG, :PND, :POT, :Q2C, :RBBT, :RIC, :SAT, :SPA, :SUN, :TAK, :TES, :TOP, 
                      :UNO, :USDE, :UTC, :ZED, :ZEIT
                    ]
  
  class << self
    def market_summary(exchange=nil)
      if exchange.is_a?(Symbol) || exchange.is_a?(String)
        exchange = exchange.to_sym.upcase
        raise ArgumentError, "You must specify a valid exchange: #{VALID_EXCHANGES}!" if !VALID_EXCHANGES.include?(exchange)
        response = JSON.parse(Mechanize.new.get("#{API_URL}/summary/#{exchange}").body)
      else
        response = JSON.parse(Mechanize.new.get("#{API_URL}/summary").body)
      end
      response.map {|c| CoinSummary.new(c) }
    end
    
    def market_stats(coin, exchange=:BTC)
      exchange = exchange.to_sym.upcase
      raise ArgumentError, "You must specify a valid exchange: #{VALID_EXCHANGES}!" if !VALID_EXCHANGES.include?(exchange)
      coin = coin.to_sym.upcase
      raise ArgumentError, "You must specify a valid coin: #{VALID_COINS}!" if !VALID_COINS.include?(coin)
      
      response = JSON.parse(Mechanize.new.get("#{API_URL}/stats/#{coin}/#{exchange}"))
      CoinSummary.new(response.first)
    end
    
    def market_trades
    #  
    end
    
    def market_orders
     # 
    end
    
    def market_chart_data
      #
    end
  end
  
  class CoinSummary < Hashie::Trash
    property :market_id, :transform_with => lambda {|v| v.to_i rescue v }
    property :code, :transform_with => lambda {|v| v.to_sym rescue v }
    property :exchange, :transform_with => lambda {|v| v.to_sym rescue v }
    property :last_price, :transform_with => lambda {|v| BigDecimal.new(v.to_s) rescue v }
    property :yesterday_price, :transform_with => lambda {|v| BigDecimal.new(v.to_s) rescue v }
    property :change, :transform_with => lambda {|v| v.to_f rescue v }
    property :highest_24h, :from => '24hhigh', :with => lambda {|v| BigDecimal.new(v.to_s) rescue v }
    property :lowest_24h, :from => '24hlow', :with => lambda {|v| BigDecimal.new(v.to_s) rescue v }
    property :volume_24h, :from => '24hvol', :with => lambda {|v| v.to_f rescue v }
    property :top_bid, :transform_with => lambda {|v| BigDecimal.new(v.to_s) rescue v }
    property :top_ask, :transform_with => lambda {|v| BigDecimal.new(v.to_s) rescue v }
  end
  
end